from .allocation import *
from .arithmetic import *
from .benchmarking import *
from .chemistry import *
from .exponentiation import *
from .finance import *
from .operators import *
from .qsvm import *
from .standard_gates import *

CORE_LIB_DECLS = [
    func.func_decl
    for func in (
        molecule_ucc,
        molecule_hva,
        molecule_hartree_fock,
        fock_hamiltonian_ucc,
        fock_hamiltonian_hva,
        fock_hamiltonian_hartree_fock,
        log_normal_finance,
        gaussian_finance,
        pauli_feature_map,
        bloch_sphere_feature_map,
        H,
        X,
        Y,
        Z,
        I,
        S,
        T,
        SDG,
        TDG,
        PHASE,
        RX,
        RY,
        RZ,
        R,
        RXX,
        RYY,
        RZZ,
        CH,
        CX,
        CY,
        CZ,
        CRX,
        CRY,
        CRZ,
        CPHASE,
        SWAP,
        IDENTITY,
        prepare_state,
        prepare_amplitudes,
        unitary,
        add,
        modular_add,
        integer_xor,
        modular_add_constant,
        real_xor_constant,
        U,
        CCX,
        allocate,
        free,
        randomized_benchmarking,
        inplace_prepare_state,
        inplace_prepare_amplitudes,
        single_pauli_exponent,
        suzuki_trotter,
        qdrift,
        exponentiation_with_depth_constraint,
    )
]

STD_QMOD_OPERATORS = [func.func_decl for func in (apply, permute)]

__all__ = [  # noqa: RUF022
    "CCX",
    "CH",
    "CPHASE",
    "CRX",
    "CRY",
    "CRZ",
    "CX",
    "CY",
    "CZ",
    "H",
    "I",
    "IDENTITY",
    "PHASE",
    "R",
    "RX",
    "RXX",
    "RY",
    "RYY",
    "RZ",
    "RZZ",
    "SDG",
    "S",
    "SWAP",
    "T",
    "TDG",
    "U",
    "X",
    "Y",
    "Z",
    "add",
    "allocate",
    "apply",
    "bloch_sphere_feature_map",
    "exponentiation_with_depth_constraint",
    "fock_hamiltonian_hartree_fock",
    "fock_hamiltonian_hva",
    "fock_hamiltonian_ucc",
    "free",
    "gaussian_finance",
    "inplace_prepare_amplitudes",
    "inplace_prepare_state",
    "integer_xor",
    "log_normal_finance",
    "modular_add",
    "modular_add_constant",
    "molecule_hartree_fock",
    "molecule_hva",
    "molecule_ucc",
    "pauli_feature_map",
    "permute",
    "prepare_amplitudes",
    "prepare_state",
    "qdrift",
    "randomized_benchmarking",
    "real_xor_constant",
    "single_pauli_exponent",
    "suzuki_trotter",
    "unitary",
]
BUILTIN_FUNCTION_DECLARATIONS = {
    func_decl.name: func_decl for func_decl in STD_QMOD_OPERATORS + CORE_LIB_DECLS
}
